<?php

//include_once("xmlrpcphp/xmlrpc.inc");
require_once dirname(__FILE__) . '/../../../../../vendor/phpxmlrpc/xmlrpc.inc';


if (!function_exists("json_decode")) {
  include_once("json/json.inc");
}

//error_reporting(E_ALL);

class edubreakUser {

  public $sipid;
  public $ecuid;
  public $name;
  public $pass;
  public $mail;
  public $picture;
  public $isModerator;

  public function __construct($sipid, $name, $pass, $mail, $picture, $isModerator=false, $ecuid = null) {
    $this->sipid = $sipid;
    $this->name = $name;
    $this->pass = $pass;
    $this->mail = $mail;
    $this->picture = $picture;
    $this->isModerator = $isModerator;
    $this->ecuid = $ecuid;
  }

}

class edubreakCourse {

  public $eccid;
  public $name;
  public $description;

  public function __construct($name, $description, $eccid = null) {
    $this->name = $name;
    $this->description = $description;
    $this->eccid = $eccid;
  }

}

/**
 * edubreak Connect functions
 */
class edubreakConnector {

  private $host;
  private $sessid;
  private $app_key;
  private $app_secret;
  private $client;
  private $connected;
  private $app;
  private $debug;

  /**
   * Constructor
   * @param <type> $host
   * @param <type> $name
   * @param <type> $pass
   * @param <type> $debug
   */
  public function __construct($host, $app_key, $app_secret, $debug=0) {
    $this->host = $host;
    $this->app_key = $app_key;
    $this->app_secret = $app_secret;

    //create the xmlrpc client
    $this->client = new xmlrpc_client($this->host . '/xmlrpc.php');
    $this->client->return_type = 'phpvals';
    $this->client->setDebug($debug < 2 ? $debug : 0);
    $this->connected = FALSE;
    $this->debug = $debug;
  }

  /**
   * Connect to edubreak
   * @return bool True if successfully authenticated
   */
  public function connect() {
    if ($this->connected) {
      $this->error("Already connected");
    }

    if ($this->_connect()) {
      $this->log("!sessid" . $this->sessid);
      if ($this->authenticate()) {
        $this->connected = TRUE;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Create a new course in edubreak and optionally adds. The optional array
   * members should be valid edubreakUser objects. Users accounts that do not
   * have an ecuid will be created. So if you use this, remeber to save the ecuids
   * in your local db.
   *
   * @param <string> $name Name of the course
   * @param <string> $description Description of the course
   * @param <array> $members Members of the course as edubreak users.
   * @return boolean True if success
   */
  public function createCourse(&$course, &$members = array()) {

    $domainStruct = array('' . $this->app['domain_id'] => new xmlrpcval($this->app['domain_id'], "int"));
    $myStruct = array(
      "type" => new xmlrpcval("course", "string"),
      "title" => new xmlrpcval($course->name, "string"),
      "status" => new xmlrpcval("1", "string"),
      "domains" => new xmlrpcval($domainStruct, "struct"),
      "domain_source" => new xmlrpcval('' . $this->app['domain_id'], "string"),
      "og_description" => new xmlrpcval($course->description, "string"),
    );

    $message = new xmlrpcmsg("node.save", array(new xmlrpcval($this->sessid, "string"), new xmlrpcval($myStruct, "struct")));
    $resp = $this->client->send($message);

    if ($this->_checkResponseIsFine($resp)) {
      $course->eccid = $resp->value();
    }
    else{
      return false;
    }

    foreach ($members as &$user) {
      $this->addMemberToCourse($course, $user);
    }
    return true;
  }

  /**
   * Adds a memeber to a course. Users with no ecuid will also be created.
   * Username and email address must be unique in edubreak, so it would be wise
   * to create a special name like $username@$yourinstitution. Otherwise the
   * connector returns a service error (401).
   *
   * If no password and mail is passed in edubreakUser, edubreakCampus will add
   * generic values and write it in the object
   *
   * @param edubreakCourse $course
   * @param edubreakUser $user
   * @return bool
   */
  public function addMemberToCourse($course, &$user) {
    $myStruct = array(
      "username" => new xmlrpcval($user->name, "string"),
      "isModerator" => new xmlrpcval($user->isModerator, "boolean"),
      "ecuid" => new xmlrpcval($user->ecuid, "int"),
      "mail" => new xmlrpcval($user->mail, "string"),
      "picture" => new xmlrpcval($user->picture, "string"),
      "pass" => new xmlrpcval($user->pass, "string"),
    );

    $message = new xmlrpcmsg("edubreak_connect.addUserToCourse", array(new xmlrpcval($this->sessid, "string"), new xmlrpcval($this->app_key, "string"), new xmlrpcval($myStruct, "struct"), new xmlrpcval($course->eccid, "int"),));
    $resp = $this->client->send($message);


    if ($this->_checkResponseIsFine($resp)) {
      $responseArray = $resp->value();

      if (is_array($responseArray)) {
        $user->ecuid = $responseArray['user']['ecuid'];
        $user->pass = $responseArray['user']['pass'];
        $user->mail = $responseArray['user']['mail'];
        return TRUE;
      } elseif (strlen($responseArray)) {
          if (strpos($responseArray, 'Username already taken') !== false) {
              $this->log('OK: got ' . $responseArray);
              return true;
          }
          $this->error('KO. Error: ' .$responseArray);
          return false;
      } else {
          return true;
      }
    }
    return FALSE;
  }

  /**
   * Create an encrypted login link for the user
   * @param edubreakUser $user
   * @return string
   */
  public function createLoginUrl($user) {

  $key = $this->app_secret;
  $input = json_encode(array($user->ecuid,$user->pass,time()));

  $key = substr(md5($key), 0, 24);
  $td = mcrypt_module_open('tripledes', '', 'ecb', '');
  $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
  mcrypt_generic_init($td, $key, $iv);
  $encrypted_data = mcrypt_generic($td, $input);
  mcrypt_generic_deinit($td);
  mcrypt_module_close($td);

  $encrypted_data = strtr(base64_encode($encrypted_data),
                    array(
                      '+' => '.',
                      '=' => '-',
                      '/' => '~'
                    )
    );
    $encrypted_data = trim(chop($encrypted_data));

    return $this->host . "/connect/" . $this->app_key . '/' . $encrypted_data;
  }






  /////
  // private functions
  /////

  /**
   * Establish connection
   * @return the session id from drupal
   */
  private function _connect() {
    $message = new xmlrpcmsg("system.connect");
    $resp = $this->client->send($message);
    if ($resp->faultCode()) {
      $this->error('KO. Error: ' . $resp->faultString());
    } else {
      //$this->log('OK: got ' . $resp->value());
      $this->log(print_r($resp->value(), true));
      $respArray = $resp->value();
      if ($respArray['sessid']) {
        $this->sessid = $respArray['sessid'];
        return TRUE;
      } else {
        $this->error("no valid session id returned");
      }
    }
    return FALSE;
  }

  /**
   *
   * @param <type> $resp
   * @return <type>
   */
  private function _checkResponseIsFine($resp) {
    if ($resp->faultCode()) {
      $this->error('KO. Error: ' . $resp->faultString());
      return false;
    } else {
      $this->log('OK: got ' . print_r($resp->value(), true));
      return true;
    }
  }

  /**
   *
   * @param <string> $name
   * @param <string> $pass
   * @return <bool>  TRUE if login was successfull
   */
  private function authenticate() {
    $message = new xmlrpcmsg("edubreak_connect.auth", array(new xmlrpcval($this->sessid, "string"), new xmlrpcval($this->app_key, "string"), new xmlrpcval($this->app_secret, "string")));


    $resp = $this->client->send($message);

    //we have to set a new session id for the user
    $respArray = $resp->value();

    if ($respArray['app']) {
      $this->app = $respArray['app'];
      $this->sessid = $respArray['sessid'];
      $this->log(print_r($this->app, true));
      return TRUE;
    }
    return FALSE;
  }

  private function log($message) {
    if ($this->debug) {
      print '<pre>' . $message . '</pre>';
    }
  }

  private function error($message) {
    print $message . '<br>';
  }

}

?>